require "thor"
require "json"
require "bbc/cosmos/tools/config/project_config"
require "bbc/cosmos/tools/config/configuration"
require "bbc/cosmos/tools/config/api"
require "pathname"

module BBC
  module Cosmos
    module Tools
      module Config
        class App < Thor
          include Thor::Actions

          def initialize(args = [], local_options = {}, config = {})
            super(args, local_options, config)
          end

          package_name "BBC Cosmos config tool"

          desc "generate PROJECT COMPONENT_ID (--cosmos_format, --env", "Generates the cosmos config for a given component id"
          method_options :cosmos_format => :boolean, :config => :string
          method_option :env, :type => :string, :default => 'int'
          def generate(project, component_id)
            begin
              config_base(options[:config])
              config = config_from(project, options[:env])

              say "\nComponent: #{component_id}\nProject: #{project}\nEnvironement: #{options[:env]}\n", :green
              if options[:cosmos_format]
                say JSON.pretty_generate(config.generate_cosmos component_id)
              else
                say JSON.pretty_generate(config.generate component_id)
              end
            rescue Exception => e
              error e.message
            end

          end

          desc "list PROJECT (--config, --env)", "Lists all components for a given project"
          method_options :config => :string
          method_option :env, :type => :string, :default => 'int'
          def list(project)
            begin
              config_base(options[:config])
              config = config_from(project, options[:env])

              say "\nComponents: \n", :green
              config.components.each do |component_id|
                say "#{component_id}", :blue
              end
            rescue Exception => e
              error e.message
            end

          end

          desc "push PROJECT KEY_PATH = nil COMPONENT = nil (--force, --config, --env)", "Push config for a component to the cosmos API"
          method_options :force => :boolean, :config => :string
          method_option :env, :type => :string, :default => 'int'
          def push(project, key_path = nil, component_id = nil)
            begin
              config_base(options[:config])
              config = config_from(project, options[:env])
              components = component_id.nil? ? config.components : [component_id]

              reply = yes?("Are you sure you want to push changes for #{components.length} components(s) to #{options[:env]}?", :blue) unless options[:force]
              if reply || reply.nil?

                components.each do |id|
                  cosmos_config = JSON.generate(config.generate_cosmos id)

                  @api = BBC::Cosmos::Tools::Config::API.new(app_config['api'], key_path)
                  response = @api.put sprintf(app_config['endpoint'], options[:env], id), cosmos_config

                  if response.success?
                    say "Configuration for component '#{id}' successfully saved", :green
                  else
                    say "There was an error saving your config: #{response.body}", :red
                  end
                end

              else
                say "Action cancelled", :red
              end

            rescue Exception => e
              error e.message
            end
          end

          private

          def error(message)
            say message, :red
            exit 1
          end

          def config_from(project, env)
            BBC::Cosmos::Tools::Config::ProjectConfig.new(@config_base, project, env)
          end

          def app_config
            @app_config ||= BBC::Cosmos::Tools::Config::Configuration::get(@config_base)['bbc']['cosmos']
          end

          def config_base(override = nil)
            @config_base = !override.nil? && File.directory?(override) ? Pathname.new(override) : Pathname.pwd.join('configs')
            raise("Config directory doesn't exist") unless File.directory?(@config_base)
          end

        end
      end
    end
  end
end
